/*************************************************************************/
/***                         ITEST.C                                   ***/
/*************************************************************************/
/*** Zweck : Beispielprogramm zur Programmierung der isel-Interrupt-   ***/
/***         schnittstelle in C                                        ***/
/*** erstellt : J.Brackenhoff, isel-automation 10/94                   ***/
/***            Microsoft C 7.0                                        ***/
/*************************************************************************/

/*== SystemIncludes =====================================================*/
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <bios.h>


/*== Pragmas fr Compiler ===============================================*/
			     /* kein Pointerchecking durch Compiler */
#pragma check_pointer( off )
			     /* kein Stackchecking durch Compiler */
#pragma check_stack( off )


/*== Typedefs ===========================================================*/
									
typedef unsigned char BYTE;  /* wir basteln uns ein Byte */
typedef BYTE BOOL;           /* wie BOOLEAN in Pascal */

			     /* wir basteln uns ein Statusregister */
typedef struct allg_status {
	       unsigned int errcode:8;
	       unsigned int saktiv :1;
	       unsigned int stop   :1;
	       unsigned int teachin:1;
	       unsigned int test   :1;
	       unsigned int hand   :1;
	       unsigned int bahn   :1;
	       unsigned int run    :1;
	       unsigned int flag7  :1; } STATUSREG;

			     /* wir basteln uns ein Fehlerregister */
typedef struct fehler_status {
		     unsigned int referenz:1;
		     unsigned int breakerr:1;
		     unsigned int hweerror:1;
		     unsigned int sweerror:1;
		     unsigned int hweinseg:1;
		     unsigned int nachlauf:1;
		     unsigned int bahnerr :1;
		     unsigned int keyerr  :1;
		     unsigned int keyinseg:1;
		     unsigned int flag9   :1;
		     unsigned int flaga   :1;
		     unsigned int flagb   :1;
		     unsigned int flagc   :1;
		     unsigned int flagd   :1;
		     unsigned int flage   :1;
		     unsigned int flagf   :1; } FEHLERREG;

			     /* wir basteln uns ein Endschalterregister */
typedef struct endschalter_status {
		     unsigned int Xmin:1;
		     unsigned int Ymin:1;
		     unsigned int Zmin:1;
		     unsigned int Amin:1;
		     unsigned int Bmin:1;
		     unsigned int Cmin:1;
		     unsigned int Dmin:1;
		     unsigned int Emin:1;
		     unsigned int Xmax:1;
		     unsigned int Ymax:1;
		     unsigned int Zmax:1;
		     unsigned int Amax:1;
		     unsigned int Bmax:1;
		     unsigned int Cmax:1;
		     unsigned int Dmax:1;
		     unsigned int Emax:1; } ENDSWITCHSTATUS;


/*== Konstanten und Makros ==============================================*/
#ifndef TRUE
#define TRUE  ( 0 == 0 )            /* wir basteln TRUE und FALSE */
#endif
#ifndef FALSE
#define FALSE ( 0 == 1 )            /* zur Arbeit mit BOOL */
#endif

#ifndef MK_FP                    /* wurde MK_FP noch nicht definiert? */
#define MK_FP(seg, ofs) ((void far *) ((unsigned long) (seg)<<16|(ofs)))
#endif

#define BEEP() _asm { \
			_asm     sub  bx, bx    /* Page 0          */ \
			_asm     mov  ax, 0E07h /* TTY bell        */ \
			_asm     int  10h       /* BIOS 10         */ \
		    }

/*== Definitionen =======================================================*/
#define GET_VERSION                 1
#define ANLAGE_RESET                2
#define ANLAGE_TEST                 3
#define LESENSTATUS                 5
#define REFFAHRT                    6
#define STARTSTOPBREAK              10
#define ABFRAGENISTPOSI             18
#define SETSEGMENTGESCH             20
#define LINEARRELNORBEW             24
#define KREISRELBEW                 28


/*== Globals ============================================================*/
union REGS regs;             /* Register fr Interruptaufrufe */

struct moves {               /* Struct fr Werte 3-Achsen */
		long x; long y; long z;
	     } xyz_werte,position;

			     /* Pointer auf Struct fr Zielpositionen */
struct moves _far *movepointer=&xyz_werte;
			     /* Pointer auf Struct fr akt. Position */
struct moves _far *pospointer=&position;

struct cmoves {              /* Struct fr Kreisvariablen */
		long e;      /* Ebene */
		long r;      /* Richtung */
		long i;      /* Mittelpunkt 1 */
		long j;      /* Mittelpunkt 2 */
		long x;      /* Endpunkt X */
		long y;      /* Endpunkt Y */
		long z;      /* Endpunkt Z */
	     } circmove;
			     /* Pointer auf Struct fr Kreisvariablen */
struct cmoves _far *cmovepointer=&circmove;
			     /* allg. Status */
STATUSREG Status ={0,0,0,0,0,0,0,0,0};
			     /* Fehlerstatus */
FEHLERREG Fehlerstatus ={1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
			     /* Hardwareendschalterstatus */
ENDSWITCHSTATUS hwe ={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
			     /* Softwareendschalterstatus */
ENDSWITCHSTATUS swe ={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
			     /* Pointer auf Statusregister */
unsigned int _far *StatusAX;
unsigned int _far *StatusBX;
unsigned int _far *StatusCX;
unsigned int _far *StatusDX;



/*== Funktions ==========================================================*/

/******************************/
/*** Tastaturpuffer lschen ***/
/******************************/
void ClrTastBuffer(void)
{                            /* sind Schreib- und Lesepointer auf den  */
			     /* Puffer gleich, so gilt dieser als leer */
  _disable();
  *(int far *) MK_FP(0x40,0x1a)=
  *(int far *) MK_FP(0x40,0x1c);
  _enable();
}

/**********************************/
/*** Aktuelle Position anzeigen ***/
/**********************************/
void ShowPosition()
{
   regs.x.ax=ABFRAGENISTPOSI;
   regs.x.bx=FP_SEG(pospointer);
   regs.x.cx=FP_OFF(pospointer);
   int86(0x78, &regs, &regs);
   *StatusAX=regs.x.ax;

   fprintf(stderr," Position:   X: %6.2f   Y: %6.2f  Z: %6.2f",
		    (position.x/1000.),(position.y/1000.),(position.z/1000.));
   putch(0x0d);
}

/******************************/
/*** Treiberstatus abfragen ***/
/******************************/
int GetStatus()
{
   regs.x.ax=LESENSTATUS;
   int86(0x78, &regs, &regs);
   *StatusAX=regs.x.ax;
   *StatusBX=regs.x.bx;
   *StatusCX=regs.x.cx;
   *StatusDX=regs.x.dx;
}

/******************************/
/*** Treiberstatus anzeigen ***/
/******************************/
int ShowStatus(BOOL komplett)
{
   GetStatus();

   fprintf(stderr,"Status :\n");
   fprintf(stderr,"Fehlercode     : %d\n",Status.errcode);
   fprintf(stderr,"Status-Segment : %d\t",Status.saktiv);
   fprintf(stderr,"Status-Stop    : %d\n",Status.stop);
   fprintf(stderr,"Status-TeachIn : %d\t",Status.teachin);
   fprintf(stderr,"Status-Test    : %d\n",Status.test);
   fprintf(stderr,"Status-Handbew.: %d\t",Status.hand);
   fprintf(stderr,"Status-Bahn    : %d\n",Status.bahn);
   fprintf(stderr,"Status-Bewegung: %d\t",Status.run);

   if(komplett)
   {
   fprintf(stderr,"\nFehlerstatus:\n");
   fprintf(stderr,"Fehler-Referenz: %d\t",Fehlerstatus.referenz);
   fprintf(stderr,"Fehler-Break   : %d\n",Fehlerstatus.breakerr);
   fprintf(stderr,"Fehler-HWE     : %d\t",Fehlerstatus.hweerror);
   fprintf(stderr,"Fehler-SWE     : %d\n",Fehlerstatus.sweerror);
   fprintf(stderr,"\nHardwareendschalterstatus:\n");
   fprintf(stderr,"Xmin: %d   Ymin: %d   Zmin: %d   Xmax: %d   Ymax: %d   Zmax: %d\n"
		 ,hwe.Xmin,hwe.Ymin,hwe.Zmin,hwe.Xmax,hwe.Ymax,hwe.Zmax);
   fprintf(stderr,"\nSoftwareendschalterstatus:\n");
   fprintf(stderr,"Xmin: %d   Ymin: %d   Zmin: %d   Xmax: %d   Ymax: %d   Zmax: %d\n"
		 ,swe.Xmin,swe.Ymin,swe.Zmin,swe.Xmax,swe.Ymax,swe.Zmax);
   }
}

/*******************************/
/*** Treiberversion abfragen ***/
/*******************************/
void GetVersion()
{
   char version[8];

   regs.x.ax=GET_VERSION;
   int86(0x78, &regs, &regs);
   version[0]=regs.h.bh;
   version[1]=regs.h.bl;
   version[2]=regs.h.ch;
   version[3]=regs.h.cl;
   version[4]=regs.h.dh;
   version[5]=regs.h.dl;
   version[6]='\0';
   fprintf( stderr,"\nTreiberversion : %s\n",version);
   *StatusAX=regs.x.ax;
}

/**************************/
/*** Treiber rcksetzen ***/
/**************************/
void AnlageReset()
{
   regs.x.ax=ANLAGE_RESET;
   int86(0x78, &regs, &regs);
   *StatusAX=regs.x.ax;
   ShowStatus(FALSE);
}

/**************************/
/*** Bewegung abbrechen ***/
/**************************/
void Abort_Move()
{
   regs.x.ax=STARTSTOPBREAK;
   regs.x.bx=3;
   int86(0x78, &regs, &regs);
   *StatusAX=regs.x.ax;
   GetStatus();
}

/**********************************/
/*** Geschwindigkeit einstellen ***/
/**********************************/
int GeschwindigkeitNormal(double g)
{
   regs.x.ax=SETSEGMENTGESCH;
   regs.x.bx=(unsigned int)((((long)(g*1000.))>>16)&0xffff);
   regs.x.cx=(unsigned int)(((long)(g*1000.))&0xffff);
   int86(0x78, &regs, &regs);
   *StatusAX=regs.x.ax;
}

/********************************/
/*** Relative Bewegung linear ***/
/********************************/
void LinearBewegung(double x, double y, double z)
{
   xyz_werte.x=(long)(x*1000.);
   xyz_werte.y=(long)(y*1000.);
   xyz_werte.z=(long)(z*1000.);
   regs.x.ax=LINEARRELNORBEW;
   regs.x.bx=FP_SEG(movepointer);
   regs.x.cx=FP_OFF(movepointer);
   int86(0x78, &regs, &regs);
   *StatusAX=regs.x.ax;

   while(Status.run)
     {
       GetStatus();
       ShowPosition();
			     /* ESC = Abbruch der Bewegung */
       if(kbhit()) if(getch()==27) Abort_Move();
     }
   putch(0x0a);
   ShowStatus(FALSE);
   ShowPosition();
}

/**********************************/
/*** Relative Bewegung circular ***/
/**********************************/
void KreisBewegung(int e, int r, double i, double j, double x, double y, double z)
{
   circmove.r=(long)r;
   circmove.e=(long)e;
   circmove.i=(long)(i*1000.);
   circmove.j=(long)(j*1000.);
   circmove.x=(long)(x*1000.);
   circmove.y=(long)(y*1000.);
   circmove.z=(long)(z*1000.);
   regs.x.ax=KREISRELBEW;
   regs.x.bx=FP_SEG(cmovepointer);
   regs.x.cx=FP_OFF(cmovepointer);
   int86(0x78, &regs, &regs);
   *StatusAX=regs.x.ax;

   while(Status.run)
     {
       GetStatus();
       ShowPosition();

			     /* ESC = Abbruch der Bewegung */
       if(kbhit()) if(getch()==27) Abort_Move();
     }
   putch(0x0a);
   ShowStatus(FALSE);
   ShowPosition();
}


/********************************/
/*** Referenzfahrt simulieren ***/
/********************************/
void SimulationReferenz()
{
   regs.x.ax=ANLAGE_TEST;
   regs.x.bx=0x0001;
   int86(0x78, &regs, &regs);
   *StatusAX=regs.x.ax;
   GetStatus();

   regs.x.ax=REFFAHRT;
   regs.x.bx=0x0007;
   int86(0x78, &regs, &regs);
   *StatusAX=regs.x.ax;
   GetStatus();

   regs.x.ax=ANLAGE_TEST;
   regs.x.bx=0x0000;
   int86(0x78, &regs, &regs);
   *StatusAX=regs.x.ax;
   ShowStatus(FALSE);
   ShowPosition();
}


/*********************/
/*** Hauptprogramm ***/
/*********************/
main()
{
  unsigned int key_entry;
  int scancode=0;
  BOOL Ende = FALSE;

   StatusAX=(unsigned int _far *)(&Status);
   StatusBX=(unsigned int _far *)(&Fehlerstatus);
   StatusCX=(unsigned int _far *)(&hwe);
   StatusDX=(unsigned int _far *)(&swe);

   fprintf(stderr,"\nINT78_Test>");
   fprintf(stderr," Rcksetzen des Treibers\n");
   AnlageReset();
   fprintf(stderr,"\nINT78_Test>");
   fprintf(stderr," Abfrage der Treiberversion\n");
   GetVersion();
   fprintf(stderr,"\nINT78_Test>");
   fprintf(stderr," Tastenbelegung:\n");
   fprintf(stderr," F1  --> Referenzfahrt simulieren\n");
   fprintf(stderr," F2  --> Lineare Bewegung\n");
   fprintf(stderr," F3  --> Kreisbewegung\n");
   fprintf(stderr," F4  --> Kompletten Status anzeigen\n");
   fprintf(stderr," ESC --> Ende\n");
   fprintf(stderr,"\nINT78_Test>");

   while(!Ende)
    {
      if(_bios_keybrd(_KEYBRD_READY)!=0)       /* Eingabe Befehle*/
	{
	  fprintf(stderr,"\n");
	  key_entry=_bios_keybrd(_KEYBRD_READ);
	  scancode=(key_entry>>8)&0xff;

	  switch(scancode)
	    {
	      case 1:                    /* ESC = Programmabbruch */
		fprintf(stderr,"\nTestprogramm abgebrochen !\n");
		Ende = TRUE;
		break;
	      case 59:                   /* F1 */
		fprintf(stderr," Referenzfahrt simulieren\n");
		SimulationReferenz();
		break;
	      case 60:                   /* F2 */
		fprintf(stderr," Bewegung Relativ Linear\n");
		GeschwindigkeitNormal(10.);
		LinearBewegung(30.,40.,20.);
		break;
	      case 61:                   /* F3 */
		fprintf(stderr," Bewegung Relativ Circular\n");
		GeschwindigkeitNormal(6.);
		KreisBewegung(0,1,10.,0.,0.,0.,0.);
		break;
	      case 62:                   /* F4 */
		fprintf(stderr," Statusanzeige komplett\n");
		ShowStatus(TRUE);
		break;
	      default:
		BEEP();
		break;
	    }
	  fprintf(stderr,"\n\nINT78_Test>");
	  ClrTastBuffer();
	}
    }
}

/*== Ende ===============================================================*/
